options  notes nocenter nodate mprint symbolgen mlogic ls=80 ps=80;

Libname HSAbun 'c:\shared\hs abundance 2005\NBvsPOI';

DM 'log;clear';
DM 'list;clear';

/* -------------------------------------------------------------------------------------------- */
/*                               CREATE SAS DATA SET FOR SSEAK 1998                             */
/* -------------------------------------------------------------------------------------------- */

PROC IMPORT OUT= HSAbun.d1 
          DATAFILE= "c:\shared\HS Abundance 2005\NBvsPOI\SSEAK98_FINAL.xls" 
          DBMS=EXCEL REPLACE;
     SHEET="Sheet1$"; 
     GETNAMES=YES;
     MIXED=NO;
     SCANTEXT=YES;
     USEDATE=YES;
     SCANTIME=YES;
RUN;

Data HSAbun.d2;
  set HSAbun.d1;
  /* change date to days from 15 August for numerical stability (keep numbers from getting too large) */
  date1 = 31*(month(date) - 8) + day(date) - 15;
  /* change time to fractional hours for numerical stability (keep numbers from getting too large) */
  time1 = hour(time) + minute(time)/60;
  /* glacial sites are analyzed differently */
  if substrate = "ice " then delete;
run;
/* create a sequential number as a site ID */
data HSAbun.site;
  set HSAbun.d2;
  keep site;
run;
proc sort data = HSAbun.site out=HSAbun.sitesort nodupkey;  by site; run;
data HSAbun.site;
  set HSAbun.sitesort;
  siteid = _n_;
run;
proc sort data = HSAbun.d2; by site; run;
proc sort data = HSAbun.site; by site; run;
/* add site ID to data set and only keep variables that are needed for the analysis */
data HSAbun.d2;
  merge HSAbun.d2 HSAbun.site;
  by site;
  count = number;
  tidehtrel = tidereltolow;
  keep count site siteid date1 time1 tidehtrel;
run;

data HSAbun.SSEAK98;
  set HSAbun.d2;
  date = date1;
  time = time1;
  drop date1 time1;
run;


/* ----------------------------------------------------------------------------------------------- */
/*                                    MACRO GLMsealTOT                                             */
/* fit an overdispersed Poisson or Negative binomial model to the data                             */
/* then adjust the counts and their totals to a set of specified covariate conditions              */
/* note that adjustments are always made to low tide in this version                               */
/* returns the adjusted total estimate                                                             */
/* writes EXCEL output files of fitted models for date, time of day and tide                       */
/* inputs to macro:                                                                                */
/*    indata: input data set                                                                       */
/*    nsites: number of sites in the data set                                                      */
/*   fixdate: the data that you want to adjust counts to                                           */
/*   fixtime: the time that you want to adjust counts to                                           */
/*  datefout: name of EXCEL output file with fitted effect of date (on logit scale)                */
/*  timefout: name of EXCEL output file with fitted effect of time of day (on logit scale)         */
/*  tidefout: name of EXCEL output file with fitted effect of tide (on logit scale)                */
/*      Dist: distribution; either Poi for overdispersed Poisson or NB for negative binomial       */
/* ----------------------------------------------------------------------------------------------- */

%macro GLMsealTOT(indata, nsites, fixdate, fixtime, datefout, timefout, tidefout, Dist);

  ods exclude Glimmix.CovB;
 
  /* OVERDISPERSED POISSON OR NEGATIVE BINOMIAL REGRESSION */
  proc glimmix data = &indata pconv=1e-6;
    class siteid;
    model count = siteid date date*date date*date*date time time*time time*time*time
       tidehtrel tidehtrel*tidehtrel tidehtrel*tidehtrel*tidehtrel/ 
          noint solution covb dist = &Dist ddfm = residual;
    %if &Dist = Poi %then %do; random _residual_; %end;
    ods output covb = covbout;
    ods output ParameterEstimates = fixsol;
	output out=glmixout pred(noblup ilink) = p 
                        resid(noblup ilink)= r 
                        variance(noblup ilink)= k;
  run;

  /* TOTAL ESTIMATE FROM ADJUSTED COUNTS */
  proc iml;
    reset nolog noname spaces=3;
    use covbout;
      read all into covb;
    nr = nrow(covb);
    covb = covb[1:nr,3:(nr+2)];
	covbtemp = covb[1:10,1:10];
    use fixsol;
      read all var{Estimate} into est;
	est = est[1:nr];
    use fixsol;
      read all var{siteid} into siteid;
    siteid = siteid[1:&nsites];
	fd1 = &fixdate;
	fd2 = (&fixdate)**2;
	fd3 = (&fixdate)**3;
	ft1 = &fixtime;
	ft2 = (&fixtime)**2;
	ft3 = (&fixtime)**3;
    L1 = I(&nsites)||
         J(&nsites,1,fd1)||J(&nsites,1,fd2)||J(&nsites,1,fd3)||
         J(&nsites,1,ft1)||J(&nsites,1,ft2)||J(&nsites,1,ft3)||
         J(&nsites,3,0);
    mu = L1*est;
    adjcnt = exp(mu);
    S = L1*covb*L1`;
    muse = sqrt(vecdiag(S));
    adjcntse = muse#exp(mu);
    tot = sum(exp(mu));
    totse = sqrt((exp(mu))`*S*exp(mu));
    actot = tot||totse;
    ac = siteid||adjcnt||adjcntse||mu||muse;
    names={"siteid" "adjusted_counts" "adj_cnts_se" "log_adj_counts" "log_adj_cnts_se"};
    create adjcount from ac [colname=names]; append from ac;
    names={"TOTAL" "Stndard Err"};
    create adjtotal from actot [colname=names]; append from actot;

    use &indata var{date};
      read all into date;
	n = nrow(date);
    use &indata var{time};
      read all into time;
    use &indata var{tidehtrel};
      read all into tide;
    use &indata var{siteid};
      read all into siteid;

	mndate = sum(date)/n;
    mntime = sum(time)/n;
    mntide = sum(tide)/n;
	mnsite = sum(est[1:&nsites,])/&nsites;

	/* FITS OVER THE RANGE OF OBSERVED VALUES */
		  /* DATE */
    daterang = (min(date) + (max(date) - min(date))*((0:100)/100))`;
	L2 = J(101,&nsites,1/&nsites)||
         daterang||(daterang##2)||(daterang##3)||
         J(101,1,mntime)||J(101,1,mntime##2)||J(101,1,mntime##3)||
         J(101,1,mntide)||J(101,1,mntide##2)||J(101,1,mntide##3);
    L20 = L2;
	L20[,(&nsites+1):(&nsites+3)] = J(101,1,mndate)||J(101,1,mndate##2)||J(101,1,mndate##3);
	L2 = L2 - L20; 
    datefit = L2*est;
	datese = sqrt(vecdiag(L2*covb*L2`));
	dateLCI = datefit - 1.96*datese;
	dateUCI = datefit + 1.96*datese;
	datefout = daterang||datefit||dateLCI||dateUCI;
    names={"date" "date_fit" "date_LO95CI" "date_UP95CI"};
    create datefout from datefout [colname=names]; append from datefout;

		  /* TIME OF DAY */
 	timerang = (min(time) + (max(time) - min(time))*((0:100)/100))`;
	L3 = J(101,&nsites,1/&nsites)||
         J(101,1,mndate)||J(101,1,mndate##2)||J(101,1,mndate##3)||
         timerang||(timerang##2)||(timerang##3)||
         J(101,1,mntide)||J(101,1,mntide##2)||J(101,1,mntide##3);
    L30 = L3;
	L30[,(&nsites+4):(&nsites+6)] = J(101,1,mntime)||J(101,1,mntime##2)||J(101,1,mntime##3);
	timefit = L3*est - L30*est;
	L3 = L3 - L30; timefit = L3*est;
	timeLCI = timefit - 1.96*sqrt(vecdiag(L3*covb*L3`));
	timeUCI = timefit + 1.96*sqrt(vecdiag(L3*covb*L3`));
	timefout = timerang||timefit||timeLCI||timeUCI;
    names={"time" "time_fit" "time_LO95CI" "time_UP95CI"};
    create timefout from timefout [colname=names]; append from timefout;

		  /* TIDE */
 	tiderang = (min(tide) + (max(tide) - min(tide))*((0:100)/100))`;
	L4 = J(101,&nsites,1/&nsites)||
         J(101,1,mndate)||J(101,1,mndate##2)||J(101,1,mndate##3)||
         J(101,1,mntime)||J(101,1,mntime##2)||J(101,1,mntime##3)||
         tiderang||(tiderang##2)||(tiderang##3);
    L40 = L4;
	L40[,(&nsites+7):(&nsites+9)] = J(101,1,mntide)||J(101,1,mntide##2)||J(101,1,mntide##3);
	tidefit = L4*est - L40*est;
	L4 = L4 - L40; tidefit = L4*est;
	tideLCI = tidefit - 1.96*sqrt(vecdiag(L4*covb*L4`));
	tideUCI = tidefit + 1.96*sqrt(vecdiag(L4*covb*L4`));
	tidefout = tiderang||tidefit||tideLCI||tideUCI;
    names={"tide" "tide_fit" "tide_LO95CI" "tide_UP95CI"};
    create tidefout from tidefout [colname=names]; append from tidefout;

quit;
run;

PROC EXPORT DATA= WORK.DATEFOUT 
            OUTFILE= "c:\shared\HS Abundance 2005\NBvsPOI\&datefout.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="sheet1"; 
RUN;
PROC EXPORT DATA= WORK.TIMEFOUT 
            OUTFILE= "c:\shared\HS Abundance 2005\NBvsPOI\&timefout.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="sheet1"; 
 RUN;
PROC EXPORT DATA= WORK.TIDEFOUT 
            OUTFILE= "c:\shared\HS Abundance 2005\NBvsPOI\&tidefout.xls" 
            DBMS=EXCEL REPLACE;
     SHEET="sheet1"; 
 RUN;

 proc print data = adjcount; run;
 proc print data = adjtotal; run;

%mend GLMsealTOT;

/* -------------------------------------------------------------------------------------------- */
/*                               ESTIMATES FOR SSEAK 1998                                       */
/* -------------------------------------------------------------------------------------------- */

/* fit the models for 3 days from 15 August at hour 10.1 */
%GLMsealTOT(indata = HSAbun.SSEAK98, nsites = 423, fixdate = 3, fixtime = 10.1,
  datefout = datefout98Poi, timefout = timefout98Poi, tidefout = tidefout98Poi,
  Dist = Poi);
PROC EXPORT DATA= WORK.Glmixout 
            OUTFILE= "C:\Shared\HS Abundance 2005\NBvsPOI\Poi.residuals070327.csv" 
            DBMS=CSV REPLACE;
RUN;

%GLMsealTOT(indata = HSAbun.SSEAK98, nsites = 423, fixdate = 3, fixtime = 10.1,
  datefout = datefout98NB, timefout = timefout98NB, tidefout = tidefout98NB,
  Dist = NB);
PROC EXPORT DATA= WORK.Glmixout 
            OUTFILE= "C:\Shared\HS Abundance 2005\NBvsPOI\NB.residuals070327.csv" 
            DBMS=CSV REPLACE;
RUN;

/* -------------------------------------------------------------------------------------------- */
/*                        SPLIT THE DATA INTO SMALL SITES AND LARGE SITES                       */
/* -------------------------------------------------------------------------------------------- */

 proc sort data = HSAbun.SSEAK98; by siteid; run;
 proc means data = HSAbun.SSEAK98;
   var count; by siteid;
   output out = HSAbun.sitemeans mean = mean;
run;

data HSAbun.idsmall;
  set HSAbun.sitemeans;
  if mean >= 33 then delete;
  siteid1 = siteid;
  keep siteid siteid1;
run;
data HSAbun.idlarge;
  set HSAbun.sitemeans;
  if mean < 33 then delete;
  siteid1 = siteid;
  keep siteid siteid1;
run;
data HSAbun.smallsites;
  merge HSAbun.SSEAK98 HSAbun.idsmall;
  by siteid;
  if siteid1 = . then delete;
run;
data HSAbun.largesites;
  merge HSAbun.SSEAK98 HSAbun.idlarge;
  by siteid;
  if siteid1 = . then delete;
run;

%GLMsealTOT(indata = HSAbun.largesites, nsites = 226, fixdate = 3, fixtime = 10.1,
  datefout = datefoutlarge, timefout = timefoutlarge, tidefout = tidefoutlarge,
  Dist = NB);
%GLMsealTOT(indata = HSAbun.smallsites, nsites = 197, fixdate = 3, fixtime = 10.1,
  datefout = datefoutsmall, timefout = timefoutsmall, tidefout = tidefoutsmall,
  Dist = NB);
%GLMsealTOT(indata = HSAbun.smallsites, nsites = 197, fixdate = 6, fixtime = 10.1,
  datefout = datefoutsmall, timefout = timefoutsmall, tidefout = tidefoutsmall,
  Dist = NB);

